package cn.zlg.www.main;

import java.util.ArrayList;
import cn.zlg.www.ControlCAN;
import cn.zlg.www.ErrData;
import cn.zlg.www.demo.R;
import cn.zlg.www.fragment.DisplayFragment;
import cn.zlg.www.fragment.SendFragment;
import cn.zlg.www.fragment.SetFragment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.view.ViewPager;
import android.support.v4.view.ViewPager.OnPageChangeListener;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends FragmentActivity {
	private ViewPager mPager;
	private ArrayList<Fragment> mFragmentsList;
	private DisplayFragment mDisplayFragment = null;
	private SendFragment mSendFragment = null;
	private SetFragment mSetFragment = null;
	private TextView mDisplayTextView, mSendTextView, mSetTextView,
			mTitleTextView;
	private ImageView mDisplayImageView, mSendImageView, mSetImageView;
	private PopupWindow mPopupWindow;
	private MoreMenuClickListener mMoreItemClickListener;

	private MsgHandler msgHandler;

	public static final int TAB_DISPLAY = 0;
	public static final int TAB_SEND = 1;
	public static final int TAB_SET = 2;
	private static int mCurTabIndex = TAB_DISPLAY;

	private static final int MENU_WITH = 150;

	public static final int TYPE_TCP_SERVER = 0;
	public static final int TYPE_TCP_CLIENT = 1;
	public static final int TYPE_UDP = 2;
	public static final int TYPE_UNCONNECT = 3;
	public static final int TYPE_CLEAR = 4;
	public static final int TYPE_SEND_FRM = 5;
	public static final int TYPE_DISPLAY_LIST = 6;
	public static final int TYPE_EXIT_APP = 7;
	public static final int TYPE_OPEN_FILE_SAVED = 8;

	private ImageView mPauseImageView;
	private ImageView mClearImageView;
	private ImageView mMoreImageView;
	private ControlCAN controlCAN;
	private boolean mPaused = true;
	private Thread mFrmRecvThread, mErrRecvThread;
	private boolean mConnected;
	private float mDensity;
	private long mExitTimeCount;
	private long mClickTimeCount;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		controlCAN = new ControlCAN();
		initView();
		mDensity = getResources().getDisplayMetrics().density;
		msgHandler = new MsgHandler();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		showMenu();

		return false;
	}

	private void initView() {
		LinearLayout dispLayout = (LinearLayout) findViewById(R.id.displaylinearLayout);
		dispLayout.setOnClickListener(new MyOnClickListener(TAB_DISPLAY));
		LinearLayout sendLayout = (LinearLayout) findViewById(R.id.sendlinearLayout);
		sendLayout.setOnClickListener(new MyOnClickListener(TAB_SEND));
		LinearLayout setLayout = (LinearLayout) findViewById(R.id.setlinearLayout);
		setLayout.setOnClickListener(new MyOnClickListener(TAB_SET));
		mDisplayTextView = (TextView) findViewById(R.id.displayTextview);
		mSendTextView = (TextView) findViewById(R.id.sendTextview);
		mSetTextView = (TextView) findViewById(R.id.setTextview);
		mSetTextView.setTextColor(getResources().getColor(
				R.color.cornflowerblue));
		mTitleTextView = (TextView) findViewById(R.id.textViewTitle);
		mDisplayImageView = (ImageView) findViewById(R.id.displayimageView);
		mDisplayImageView.setSelected(true);
		mSendImageView = (ImageView) findViewById(R.id.sendimageView);
		mSetImageView = (ImageView) findViewById(R.id.setimageView);

		mPauseImageView = (ImageView) findViewById(R.id.imageViewzPause);
		mPauseImageView.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View arg0) {
				// TODO Auto-generated method stub
				if (System.currentTimeMillis() - mClickTimeCount < 1000) {
					return;
				}
				if (mConnected) {
					mPaused = !mPaused;
					mSendFragment.setSendStatus(mPaused);
					if (!mPaused) {
						startRecvData();
					}

					mPauseImageView
							.setImageResource(mPaused ? android.R.drawable.ic_media_play
									: android.R.drawable.ic_media_pause);
				} else {
					Toast toast = Toast.makeText(MainActivity.this, "δ",
							Toast.LENGTH_SHORT);
					toast.setGravity(Gravity.CENTER, 0, 0);
					toast.show();
				}
				mClickTimeCount = System.currentTimeMillis();
			}
		});
		mClearImageView = (ImageView) findViewById(R.id.imageViewdelete);
		mClearImageView.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				// TODO Auto-generated method stub
				if (mDisplayFragment != null) {
					AlertDialog clearDlg = new AlertDialog.Builder(
							MainActivity.this)
							.setIcon(android.R.drawable.ic_delete)
							.setMessage("б?")
							.setPositiveButton("",
									new DialogInterface.OnClickListener() {
										@Override
										public void onClick(
												DialogInterface arg0, int arg1) {
											// TODO Auto-generated method stub
											mDisplayFragment.dealItem(
													DisplayFragment.MSG_CLEAR,
													null);
										}
									})
							.setNegativeButton("ȡ",
									new DialogInterface.OnClickListener() {
										@Override
										public void onClick(
												DialogInterface arg0, int arg1) {
											// TODO Auto-generated method stub
										}
									}).create();
					clearDlg.show();
				}
			}
		});
		mMoreImageView = (ImageView) findViewById(R.id.imageViewMore);
		mMoreImageView.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				showMenu();
			}
		});
		mMoreItemClickListener = new MoreMenuClickListener();

		mPager = (ViewPager) findViewById(R.id.vPager);
		mFragmentsList = new ArrayList<Fragment>();

		mDisplayFragment = new DisplayFragment();
		mFragmentsList.add(mDisplayFragment);
		mSendFragment = new SendFragment();
		mFragmentsList.add(mSendFragment);
		mSetFragment = new SetFragment();
		mFragmentsList.add(mSetFragment);

		mPager.setAdapter(new MyFragmentPagerAdapter(
				getSupportFragmentManager(), mFragmentsList));
		mPager.setOnPageChangeListener(new MyOnPageSelectListener());
		mPager.setCurrentItem(TAB_SET);
	}

	private void exitApp() {
		mSetFragment.saveSetInfo();
		clearThread();
		finish();
	}

	private void showMenu() {
		if (mPopupWindow != null && mPopupWindow.isShowing()) {
			mPopupWindow.dismiss();
			mPopupWindow = null;
		} else {
			initMorePopupWindow();
			int nX = (int) ((25 - MENU_WITH) * mDensity);
			mPopupWindow.showAsDropDown(mMoreImageView, nX, 0);
		}
	}

	public void doCommand(int nType, Object obj) throws InterruptedException {
		switch (nType) {
		case TYPE_TCP_SERVER: {
			SetInfo setInfo = (SetInfo) obj;
			if (!controlCAN.initTCPServer(setInfo.nLocalPort,
					setInfo.ipAddress, setInfo.nPortErrNotice)) {
				Toast toast = Toast.makeText(this, "ʼʧ",
						Toast.LENGTH_SHORT);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
				mSetFragment.setSureButtonFlag(false);
				mPaused = true;
				return;
			}
			connect();
			break;
		}
		case TYPE_TCP_CLIENT: {
			SetInfo setInfo = (SetInfo) obj;
			if (!controlCAN.initTCPClient(setInfo.ipAddress, setInfo.nWorkPort,
					setInfo.nPortErrNotice)) {
				Toast toast = Toast.makeText(this, "ʼͻʧ",
						Toast.LENGTH_SHORT);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
				mSetFragment.setSureButtonFlag(false);
				mPaused = true;
				return;
			}
			connect();
			break;
		}
		case TYPE_UDP: {
			SetInfo setInfo = (SetInfo) obj;
			mConnected = true;
			if (!controlCAN.initUDP(setInfo.nLocalPort, setInfo.ipAddress,
					setInfo.nWorkPort, setInfo.nPortErrNotice)) {
				Toast toast = Toast.makeText(this, "ʼʧ",
						Toast.LENGTH_SHORT);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
				mSetFragment.setSureButtonFlag(false);
				mPaused = true;
				return;
			}
			connect();
			break;
		}
		case TYPE_UNCONNECT: {
			if (controlCAN.close()) {
				unConnect();
				Toast toast = Toast
						.makeText(this, "Ͽӳɹ", Toast.LENGTH_SHORT);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
			} else {
				mSetFragment.setSureButtonFlag(true);
				Toast toast = Toast
						.makeText(this, "Ͽʧ", Toast.LENGTH_SHORT);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
			}
			break;
		}
		case TYPE_CLEAR: {
			mDisplayFragment.dealItem(DisplayFragment.MSG_CLEAR, null);
			break;
		}
		case TYPE_SEND_FRM: {
			if (mPaused || !mConnected) {
				return;
			}
			ItemData itemData = (ItemData) obj;
			mDisplayFragment.dealItem(DisplayFragment.MSG_SEND, itemData);
			ItemData[] itemDatas = new ItemData[1];
			itemDatas[0] = itemData;
			if (!controlCAN.isConnected()) {
				Message message = new Message();
				message.what = TYPE_UNCONNECT;
				msgHandler.sendMessage(message);
				return;
			}
			controlCAN.send(itemDatas, 1);
			break;
		}
		case TYPE_DISPLAY_LIST:
			mDisplayFragment.refreshRecvList();
			break;
		case TYPE_EXIT_APP:
			exitApp();
			break;
		case TYPE_OPEN_FILE_SAVED: {
			Message message = new Message();
			message.what = TYPE_OPEN_FILE_SAVED;
			message.obj = obj;
			msgHandler.sendMessage(message);
			break;
		}
		default:
			break;
		}
	}

	private void connect() {
		mConnected = true;
		mPaused = false;
		mPauseImageView.setImageResource(android.R.drawable.ic_media_pause);
		mSetFragment.setSureButtonFlag(mConnected);
		mSendFragment.setSendStatus(mPaused);
		Toast toast = Toast.makeText(this, "ӳɹ", Toast.LENGTH_SHORT);
		toast.setGravity(Gravity.CENTER, 0, 0);
		toast.show();
		setTitle(mConnected);
		startRecvData();
	}

	private void unConnect() {
		mPaused = true;
		mPauseImageView.setImageResource(android.R.drawable.ic_media_play);
		mSetFragment.setSureButtonFlag(false);
		mSendFragment.setSendStatus(true);
		mConnected = false;
		setTitle(mConnected);
	}

	private void setTitle(boolean bConnected) {
		mTitleTextView.setText(bConnected ? "CANWifi[]" : "CANWifi[δ]");
	}

	public class MyOnClickListener implements View.OnClickListener {
		private int index = 0;

		public MyOnClickListener(int i) {
			index = i;
		}

		@Override
		public void onClick(View v) {
			if (setCurTabIndex(index)) {
				mPager.setCurrentItem(index);
			}
		}
	};

	public class MyOnPageSelectListener implements OnPageChangeListener {

		@Override
		public void onPageScrollStateChanged(int arg0) {
			// TODO Auto-generated method stub

		}

		@Override
		public void onPageScrolled(int arg0, float arg1, int arg2) {
			// TODO Auto-generated method stub

		}

		@Override
		public void onPageSelected(int arg0) {
			// TODO Auto-generated method stub
			switch (arg0) {
			case TAB_DISPLAY:
				setCurTabIndex(TAB_DISPLAY);
				mDisplayFragment.refreshRecvList();
				setDisplayToolBarStatus(true);
				mDisplayImageView.setSelected(true);
				mSendImageView.setSelected(false);
				mSetImageView.setSelected(false);
				mDisplayTextView.setTextColor(getResources().getColor(
						R.color.cornflowerblue));
				mSendTextView.setTextColor(getResources()
						.getColor(R.color.gray));
				mSetTextView
						.setTextColor(getResources().getColor(R.color.gray));
				break;
			case TAB_SEND:
				setCurTabIndex(TAB_SEND);
				setDisplayToolBarStatus(false);
				mDisplayImageView.setSelected(false);
				mSendImageView.setSelected(true);
				mSetImageView.setSelected(false);
				mDisplayTextView.setTextColor(getResources().getColor(
						R.color.gray));
				mSendTextView.setTextColor(getResources().getColor(
						R.color.cornflowerblue));
				mSetTextView
						.setTextColor(getResources().getColor(R.color.gray));
				break;
			case TAB_SET:
				setCurTabIndex(TAB_SET);
				setDisplayToolBarStatus(false);
				mDisplayImageView.setSelected(false);
				mSendImageView.setSelected(false);
				mSetImageView.setSelected(true);
				mDisplayTextView.setTextColor(getResources().getColor(
						R.color.gray));
				mSendTextView.setTextColor(getResources()
						.getColor(R.color.gray));
				mSetTextView.setTextColor(getResources().getColor(
						R.color.cornflowerblue));
				break;
			default:
				break;
			}
		}

	}

	private void setDisplayToolBarStatus(boolean bVisiable) {
		if (bVisiable) {
			mClearImageView.setVisibility(View.VISIBLE);
			// mPauseImageView.setVisibility(View.VISIBLE);
		} else {
			mClearImageView.setVisibility(View.GONE);
			// mPauseImageView.setVisibility(View.GONE);
		}
	}

	private class FrmRecvRunnable implements Runnable {

		@Override
		public void run() {
			// TODO Auto-generated method stub
			int nCount = 0;
			while (!mPaused) {
				if (mConnected && mDisplayFragment != null) {
					if (!controlCAN.isConnected()) {
						Message message = new Message();
						message.what = TYPE_UNCONNECT;
						msgHandler.sendMessage(message);
						return;
					}
					if ((nCount = controlCAN.getReceiveNum()) <= 0) {
						try {
							Thread.sleep(1);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						continue;
					}
					ItemData[] objs = new ItemData[nCount];
					for (int i = 0; i < nCount; i++) {
						objs[i] = new ItemData();
					}
					if (controlCAN.recv(objs, nCount) != nCount) {
						continue;
					}
					for (int i = 0; i < nCount; i++) {
						mDisplayFragment.dealItem(DisplayFragment.MSG_RECV,
								objs[i]);
					}
				}
			}
		}
	}

	private class ErrInfoRunnable implements Runnable {

		@Override
		public void run() {
			// TODO Auto-generated method stub
			ErrData errData = new ErrData();
			while (!mPaused) {
				if (mConnected && mDisplayFragment != null) {
					if (controlCAN.readErrInfo(errData)) {
						mDisplayFragment.dealItem(DisplayFragment.ERR_DATA,
								new ErrData(errData));
					} else {
						try {
							Thread.sleep(100);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}

	}

	public int getCurTabIndex() {
		return mCurTabIndex;
	}

	boolean setCurTabIndex(int nIndex) {
		if (mCurTabIndex == nIndex)
			return false;
		mCurTabIndex = nIndex;
		if (mCurTabIndex != TAB_DISPLAY) {
			mDisplayFragment.setListAdapterEmpty();
		} else {
			mDisplayFragment.setListAdapter();
		}
		return true;
	}

	public void clearThread() {
		mPaused = true;
		if (controlCAN != null && mConnected) {
			mConnected = false;
			controlCAN.close();
		}
	}

	private void initMorePopupWindow() {
		View moreView = getLayoutInflater().inflate(R.layout.more_menu, null);
		int nWidth = (int) (mDensity * MENU_WITH);
		mPopupWindow = new PopupWindow(moreView, nWidth,
				LinearLayout.LayoutParams.WRAP_CONTENT);
		mPopupWindow.setBackgroundDrawable(getResources().getDrawable(
				R.drawable.plus));

		LinearLayout closeLayout = (LinearLayout) moreView
				.findViewById(R.id.moremenucloselayout);
		closeLayout.setOnClickListener(mMoreItemClickListener);
		LinearLayout aboutLayout = (LinearLayout) moreView
				.findViewById(R.id.moremenuaboutlayout);
		aboutLayout.setOnClickListener(mMoreItemClickListener);

		mPopupWindow.setFocusable(true);
		mPopupWindow.setOutsideTouchable(true);
		mPopupWindow.update();
	}

	private void startRecvData() {
		if (!mConnected) {
			return;
		}
		mPaused = false;
		mFrmRecvThread = new Thread(new FrmRecvRunnable());
		mFrmRecvThread.start();
		mErrRecvThread = new Thread(new ErrInfoRunnable());
		mErrRecvThread.start();
	}

	@Override
	protected void finalize() throws Throwable {
		// TODO Auto-generated method stub
		if (mSetFragment != null) {
			mSetFragment.saveSetInfo();
		}
		clearThread();
		super.finalize();
	}

	static {
		System.loadLibrary("CANWifi");
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		// TODO Auto-generated method stub
		if (KeyEvent.KEYCODE_BACK == keyCode) {
			if ((System.currentTimeMillis() - mExitTimeCount) > 2000) {
				Toast toast = Toast.makeText(this, "ٰһ˳Ӧ",
						Toast.LENGTH_SHORT);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
				mExitTimeCount = System.currentTimeMillis();
			} else {
				exitApp();
			}
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}

	private class MoreMenuClickListener implements OnClickListener {

		@Override
		public void onClick(View v) {
			// TODO Auto-generated method stub
			mPopupWindow.dismiss();
			mPopupWindow = null;
			switch (v.getId()) {
			case R.id.moremenuaboutlayout:
				View view = getLayoutInflater().inflate(R.layout.about, null);
				new AlertDialog.Builder(MainActivity.this).setView(view).show();
				break;
			case R.id.moremenucloselayout:
				exitApp();
				break;
			default:
				break;
			}
		}
	}

	private class MsgHandler extends Handler {

		@Override
		public void handleMessage(Message msg) {
			// TODO Auto-generated method stub
			switch (msg.what) {
			case TYPE_UNCONNECT:
				unConnect();
				break;
			}
		}
	}
}
